@@ -34,6 +34,12 @@ class TestSelect(unittest.TestCase):
             'SELECT * FROM "t" AS "a" WHERE ("a"."c" = %s)')
         self.assertEqual(tuple(query.params), ('foo',))
 
+    def test_select_iter(self):
+        query = self.table.select()
+        self.assertEqual(
+            tuple(query),
+            ('SELECT * FROM "t" AS "a"', ()))
+
     def test_select_without_from(self):
         query = Select([Literal(1)])
         self.assertEqual(str(query), 'SELECT %s')
@@ -49,6 +55,14 @@ class TestSelect(unittest.TestCase):
         self.assertEqual(str(query), 'SELECT (SELECT %s) AS "foo"')
         self.assertEqual(tuple(query.params), (1,))
 
+    def test_select_invalid_column(self):
+        with self.assertRaises(ValueError):
+            Select(['foo'])
+
+    def test_select_invalid_where(self):
+        with self.assertRaises(ValueError):
+            self.table.select(where='foo')
+
     def test_select_distinct(self):
         query = self.table.select(self.table.c, distinct=True)
         self.assertEqual(
@@ -68,6 +82,10 @@ class TestSelect(unittest.TestCase):
             'SELECT DISTINCT ON ("a"."a", "a"."b") "a"."c" FROM "t" AS "a"')
         self.assertEqual(tuple(query.params), ())
 
+    def test_select_invalid_distinct_on(self):
+        with self.assertRaises(ValueError):
+            self.table.select(self.table.c, distinct_on='foo')
+
     def test_select_from_list(self):
         t2 = Table('t2')
         t3 = Table('t3')
@@ -231,6 +249,10 @@ class TestSelect(unittest.TestCase):
             'GROUP BY CUBE ("a"."a", "a"."b")')
         self.assertEqual(tuple(query.params), ('*',))
 
+    def test_select_invalid_group_by(self):
+        with self.assertRaises(ValueError):
+            self.table.select(group_by=['foo'])
+
     def test_select_having(self):
         col1 = self.table.col1
         col2 = self.table.col2
@@ -241,6 +263,10 @@ class TestSelect(unittest.TestCase):
             'HAVING (MIN("a"."col2") > %s)')
         self.assertEqual(tuple(query.params), (3,))
 
+    def test_select_invalid_having(self):
+        with self.assertRaises(ValueError):
+            self.table.select(having='foo')
+
     def test_select_order(self):
         c = self.table.c
         query = self.table.select(c, order_by=Literal(1))
@@ -248,18 +274,22 @@ class TestSelect(unittest.TestCase):
             'SELECT "a"."c" FROM "t" AS "a" ORDER BY %s')
         self.assertEqual(tuple(query.params), (1,))
 
+    def test_select_invalid_order(self):
+        with self.assertRaises(ValueError):
+            self.table.select(order_by='foo')
+
     def test_select_limit_offset(self):
         try:
             Flavor.set(Flavor(limitstyle='limit'))
             query = self.table.select(limit=50, offset=10)
             self.assertEqual(str(query),
-                'SELECT * FROM "t" AS "a" LIMIT 50 OFFSET 10')
-            self.assertEqual(tuple(query.params), ())
+                'SELECT * FROM "t" AS "a" LIMIT %s OFFSET %s')
+            self.assertEqual(tuple(query.params), (50, 10))
 
             query.limit = None
             self.assertEqual(str(query),
-                'SELECT * FROM "t" AS "a" OFFSET 10')
-            self.assertEqual(tuple(query.params), ())
+                'SELECT * FROM "t" AS "a" OFFSET %s')
+            self.assertEqual(tuple(query.params), (10,))
 
             query.offset = 0
             self.assertEqual(str(query),
@@ -280,24 +310,32 @@ class TestSelect(unittest.TestCase):
 
             query.offset = 10
             self.assertEqual(str(query),
-                'SELECT * FROM "t" AS "a" LIMIT -1 OFFSET 10')
-            self.assertEqual(tuple(query.params), ())
+                'SELECT * FROM "t" AS "a" LIMIT -1 OFFSET %s')
+            self.assertEqual(tuple(query.params), (10,))
         finally:
             Flavor.set(Flavor())
 
+    def test_select_invalid_limit(self):
+        with self.assertRaises(ValueError):
+            self.table.select(limit='foo')
+
+    def test_select_invalid_offset(self):
+        with self.assertRaises(ValueError):
+            self.table.select(offset='foo')
+
     def test_select_offset_fetch(self):
         try:
             Flavor.set(Flavor(limitstyle='fetch'))
             query = self.table.select(limit=50, offset=10)
             self.assertEqual(str(query),
                 'SELECT * FROM "t" AS "a" '
-                'OFFSET (10) ROWS FETCH FIRST (50) ROWS ONLY')
-            self.assertEqual(tuple(query.params), ())
+                'OFFSET (%s) ROWS FETCH FIRST (%s) ROWS ONLY')
+            self.assertEqual(tuple(query.params), (10, 50))
 
             query.limit = None
             self.assertEqual(str(query),
-                'SELECT * FROM "t" AS "a" OFFSET (10) ROWS')
-            self.assertEqual(tuple(query.params), ())
+                'SELECT * FROM "t" AS "a" OFFSET (%s) ROWS')
+            self.assertEqual(tuple(query.params), (10,))
 
             query.offset = 0
             self.assertEqual(str(query),
@@ -390,6 +428,10 @@ class TestSelect(unittest.TestCase):
             'SELECT "a"."c" FROM "t" AS "a" FOR UPDATE')
         self.assertEqual(tuple(query.params), ())
 
+    def test_select_invalid_for(self):
+        with self.assertRaises(ValueError):
+            self.table.select(for_=['foo'])
+
     def test_copy(self):
         query = self.table.select()
         copy_query = deepcopy(query)
@@ -472,17 +514,41 @@ class TestSelect(unittest.TestCase):
             'WINDOW "b" AS (PARTITION BY "a"."c2")')
         self.assertEqual(tuple(query.params), (1,))
 
+    def test_window_with_alias(self):
+        query = self.table.select(
+            Min(self.table.c1, window=Window([self.table.c2])).as_('min'))
+
+        self.assertEqual(
+            str(query),
+            'SELECT MIN("a"."c1") OVER "b" AS "min" FROM "t" AS "a" '
+            'WINDOW "b" AS (PARTITION BY "a"."c2")')
+        self.assertEqual(query.params, ())
+
+    def test_select_invalid_window(self):
+        with self.assertRaises(ValueError):
+            self.table.select(windows=['foo'])
+
     def test_order_params(self):
         with_ = With(query=self.table.select(self.table.c,
                 where=(self.table.c > 1)))
-        w = Window([Literal(8)])
+        w = Window([Literal(7)])
         query = Select([Literal(2), Min(self.table.c, window=w)],
             from_=self.table.select(where=self.table.c > 3),
             with_=with_,
             where=self.table.c > 4,
             group_by=[Literal(5)],
-            order_by=[Literal(6)],
-            having=Literal(7))
+            having=Literal(6),
+            order_by=[Literal(8)])
+        self.assertEqual(
+            str(query),
+            'WITH "c" AS (SELECT "a"."c" FROM "t" AS "a" WHERE ("a"."c" > %s))'
+            ' SELECT %s, MIN("a"."c") OVER "b" '
+            'FROM SELECT * FROM "t" AS "a" WHERE ("a"."c" > %s) '
+            'WHERE ("a"."c" > %s) '
+            'GROUP BY %s '
+            'HAVING %s '
+            'WINDOW "b" AS (PARTITION BY %s) '
+            'ORDER BY %s')
         self.assertEqual(tuple(query.params), (1, 2, 3, 4, 5, 6, 7, 8))
 
     def test_no_as(self):
